/* hardware specific audio interface for SUN (SAU)
 * This interface is going to change !
 */

#include <sys/signal.h>
#include <sys/ioctl.h>
#ifdef SOLARIS
#include <sys/types.h>
#include <sys/audioio.h>
#else
#include <sun/audioio.h>
#endif
#include <stdio.h>
#include "synth.h"
#include "devaudio.h"

#define AUDIO_BADFD	(-1)	/* invalid audio descriptor */

/* audio state passed to audio routines */

struct audio_cookie {
	int fd;		/* file descriptor for audio */
	int hdw;		/* hardware type (0=unknown, 1=amd 2=spkrbox */
	int bps;		/* bytes per sample (shift code)*/
	int cnt;		/* # of opens */
	int wrote;	/* # of samples written */
	};

/* we only have one instance of the audio device, so make it static */

static struct audio_cookie state_data = {
	AUDIO_BADFD, 0, 0, 0, 0
	};

static void catch();
static int get_modes(int fd);
char *getenv();
int *audio_rates;		/* pointer to valid sampling rates */

/* valid audio rates - this is system dependent */

static int linear_rates[] = {
	8000, 9600, 11025, 16000, 18900, 22050, 32000, 0
	};
static int mu_rates[] = {
	8000, 0
	};

/* try to open the audio device.  If its busy, return -1 */

char *
audio_open()
	{
	int audio_fd;
	struct audio_cookie *state = &state_data;
	signal(SIGALRM,catch);
	alarm(3);
	audio_fd = open(getenv("AUDIO")?getenv("AUDIO"):"/dev/audio",1);	
	alarm(0);
	if (audio_fd < 0)
		return(NULL);

	state->wrote = 0;
	state->fd = audio_fd;
	state->hdw = get_modes(audio_fd);
	state->bps = state->hdw&2 ? 2 : 0;
	return((char *) state);
	}

/* close the audio device */

int
audio_close(cookie)
char *cookie;
	{
	struct audio_cookie *state = (struct audio_cookie *) cookie;
	
	if (state->fd != AUDIO_BADFD) {
		close(state->fd);
		state->fd = AUDIO_BADFD;
		return(1);
		}	
	else
		return(0);
	}

/* write data to audio device */

int
audio_write(cookie,buff,bytes)
char *cookie;	/* file descriptor */
char *buff;	/* pointer to samples */
int bytes;	/* number of samples */
	{
	struct audio_cookie *state = (struct audio_cookie *) cookie;
	int n = write(state->fd,buff,bytes);

	if (n>0) {
		state->wrote += n >> state->bps ;
		}
	return(n);
	}

/* return # of samples written but not yet played */

int
audio_inq(cookie)
char *cookie;
   {
	struct audio_cookie *state = (struct audio_cookie *) cookie;
   int n;
   struct audio_info info;

   n = ioctl(state->fd,AUDIO_GETINFO,&info);
	if (n<0)
		return(-1);
	else {
		return(state->wrote - info.play.samples);
		}
   }

/* return valid audio modes */

int
audio_modes(cookie)
char *cookie;
	{
	struct audio_cookie *state = (struct audio_cookie *) cookie;
	return(state->hdw);
	}

/* set the audio sampling rate, # of channels */

int
audio_setmode(cookie,rate,chan,mode)
char *cookie;	/* state cookie */
int rate;		/* samples/second */
int chan;		/* number of channels */
int mode;		/* sample size, format */
	{
	struct audio_cookie *state = (struct audio_cookie *) cookie;

#ifdef AUDIO_GETDEV

	audio_info_t s;

	if (state->hdw <=1)		/* No choices, don't bother */
		return(0);

	AUDIO_INITINFO(&s);
	dprintf('m')(stderr,"Setting rate to %d\n",rate);
	if (mode&2) {		/* 16 bit linear stereo - use line-out only */
		s.play.sample_rate = rate;
		s.play.channels    = 2;
		s.play.precision   = 16;
		s.play.encoding    = AUDIO_ENCODING_LINEAR;
		state->bps = 2;
		if (state->cnt == 0)
			s.play.port = AUDIO_LINE_OUT;
		}
	else {				/* 8 bit mu-law - use internal speaker */
		s.play.channels  = 1;
		s.play.precision = 8;
		s.play.encoding  = AUDIO_ENCODING_ULAW;
		if (state->cnt ==0) s.play.port = AUDIO_SPEAKER;
		state->bps = 0;
		}
	state->cnt++;
	ioctl(state->fd,AUDIO_SETINFO,&s);
	return(2);
#else
	return(1);
#endif
	}

/* return mask of valid audio types:
 * 1: 8-bit mu-law mono 8khz
 * 2: 16 bit linear interleaved stereo
 * 0: no know valid modes
 */

#ifdef SOLARIS

/* 'seems like this interface changes on a daily basis some times
 * This one is the pits!
 */

static int
get_modes(fd)
int fd;		/* file desciptor of audio device */
	{
	struct audio_device type;
	int result;
	ioctl(fd,AUDIO_GETDEV,&type);
	fprintf(stderr,"AUDIO: name=%s\nversion=%s\nconfig=%s\n",
			type.name,type.version,type.config);
	if (strcmp("speakerbox",type.config)==0) {
		result = AUDIO_MU|AUDIO_LIN;
		}
	else {
		result = AUDIO_MU;
		}
	audio_rates = (result&AUDIO_LIN) ?linear_rates : mu_rates;
	return(result);
	exit(0);
	}
	
#else

static int
get_modes(fd)
int fd;		/* file desciptor of audio device */
	{

#ifndef AUDIO_GETDEV

	/* pre-4.1.3, amd codec only */
	audio_rates = mu_rates;	/* default */
	return(AUDIO_MU);

#else

	int result;
	int type = 0;
	ioctl(fd,AUDIO_GETDEV,&type);
	dprintf('m')(stderr,"audio hardware type %d\n",type);
	switch(type) {
		case AUDIO_DEV_AMD:			result = AUDIO_MU; break;
		case AUDIO_DEV_SPEAKERBOX:	result = AUDIO_MU|AUDIO_LIN; break;
		case -1:		/* unknown */	result = 0; break;
		case 0:		/* unknown */	result = 0; break;
		default:		/* hope */		result = AUDIO_MU|AUDIO_LIN; break;
		}
	audio_rates = (result&AUDIO_LIN) ?linear_rates : mu_rates;
	return(result);
#endif
	}
#endif

/* catch (and ignore) a signal */

static void
catch()
	{
	fprintf(stderr,"Synth, Timeout error\n");
	}
