#!/home/suhler/bin/blt_wish -f
# new STEREO synth interface (sau)

set prop(border) 4
set prop(relief) ridge
set prop(tuning) 0
set prop(file) ""
# set prop(dir) [pwd]
set prop(dir) "./sample_tunes"
set prop(config) $env(HOME)/.voices
cd $prop(dir)

set limits(volume) "0 1000 4000"
set limits(style) "20 100 250"
set limits(transposition) "-24 0 24"
set limits(tempo) "40 100 260"
set limits(intensity) "0 5 10"

# command codes for synth

set cmd(volume) g
set cmd(style) l
set cmd(transposition) t
set cmd(tempo) T
set cmd(intensity) i

set message "message line"		;# message line
set status "stopped"				;# play status
set note "0"						;# note number
set next ""							;# next command to send to synth
set channels "0000"				;# currently active channels
set rate 16000						;# sampling rate, currently set at startup only
set synth "./synth"
set control ""						;# fd

# process command line args

if {[lsearch $argv "-v"] >=0 } {set verbose on} {set verbose ""}
if {[lsearch $argv "-d"] >=0 } {set debug on} {set debug ""}

# global settings

wm title . "new synth"
wm minsize . 10 10

# get a new file to play

proc get_file {} {
	global file prop channels
	if {[file isdirectory $file]} {
		cd $file
		set prop(dir) [pwd]
		do_message "Setting Current directory to $prop(dir)"
		set file ""
	} else {
		do_message "Selected file $file"
		queue F "$prop(dir)/$file"
		queue +
		reset_chan
		}
	}

# toggle status

proc toggle_status {win} {
	global status channels
	$win configure -relief sunken
	if {$status == "running"} {
		set status "stopped"
		reset_chan
		queue -
	}	else {
		set status "running"
		queue +
	}
}

# queue up another command

proc queue {code {value ""}} {
	global next
	if {$value == ""} {append next "$code;"} {append next "$code=$value;"}
	# puts stderr "queue: $next"
	}

# sent a command to the synthesizer periodically

proc cycle {} {
   global next

   send_synth $next
   set next ""
   after 200 cycle
   }

#  run a command from a remote interpreter, return info string

proc remote {cmd msg} {
	global control

	if {$msg != ""} {do_message "remote command: $msg"}
   puts $control $cmd
	if {[catch "flush $control"]} {die "Remote failed: Synthesizer died, sorry"}
	return [lindex [gets $control] 4]
	}

proc die {msg} {
	global control
	catch {
		puts $control "Q"
		flush $control
		close $control
		}
	if {$msg == ""} {exit 0}
	toplevel .die
	label .die.l -text $msg -fg red
	blt_table .die .die.l 0,0
	update
	after 6000
	exit 1
	}

# manage a transaction with the synthesizer (s is the message)

proc send_synth {s} {
   global control note status active channels

	# if {$s != ""} {do_message "sending: $s"}
   puts $control "$s"
	if {[catch "flush $control"]} {
		die "Flush failed $control-><$s>: Synthesizer died, sorry"
	}	

	# get results

   if {[gets $control result] == -1} {
		die "Read field: Synthesizer died, sorry"
		}
	# puts stderr "got: $result"
	set indx 0
	set last_channels $channels
	foreach i {state note active channels msg} {
		set $i [lindex $result $indx]
		incr indx
		}
	if {"$state" != "*"} {
		die "synth error: $result"
		}

	if {$active == "+"} {
		set status running
	} else {
		set status stopped
	}

	if {$msg != ""} {do_message "$msg"}

	# re-set channel mask

	if {$channels == 0000 && $status == "running"} {
		set channels $last_channels
	} else {
		if {$status == "running"} {fix_chan $channels $last_channels}
	}
}

# fix channel mask (the hard way)

proc fix_chan {new old} {
	if {$old == $new} return
	set change [expr "0xFFFF & (0x$old ^ 0x$new)"]
	foreach i {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16} {
		if {[expr "0x$change & 0x1"]} {
			if {[expr "0x$new & 0x1"]} {set color green} {set color red}
			.main.chan.$i configure -bg $color
			.main.chan.$i.s configure -sliderforeground $color
		}
		set change [expr "0xffff & (0x$change >> 1)"]
		set new [expr "0x$new >> 1"]
		if {$change == 0} break
	}
}

# reset all channels to red 

proc reset_chan  {} {
	global channels

	set channels 0000
	foreach i {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16} {
		.main.chan.$i configure -bg red
		.main.chan.$i.s configure -sliderforeground red
		}
	}

proc do_message {{s ""}} {
	global message
	set message $s
	if {"$message" != ""} {after 3000 do_message}
	}

# file completion - invoked with tab  or button-1

proc comp_file {win} {
	global file
	set files [glob -nocomplain $file*.mpu]
	if {[llength $files] <=1} {
		set file [lindex "$files $file" 0] 
	} else {
		file_list $win $files 
	}
}

# file completion, pick from a list of files

proc file_list {win files} {
	catch {destroy .f}
	toplevel .f
	wm title .f file
	wm geometry .f +[winfo rootx $win]+[winfo rooty $win]
	listbox .f.l -relief groove -border 2 -setgrid 1
	tk_listboxSingleSelect .f.l
	eval ".f.l insert 0 [lsort $files]"
	blt_table .f .f.l 0,0
	bind .f.l <Return> "get_list %W"
	focus .f.l
}

proc get_list win {
	global file
	set got [.f.l curselection]
	if {[llength $got] == "1"} {
		set file [$win get $got]
		do_message "selected file $file"
		get_file
	}
	catch {destroy .f}
	return
}

# generate menu from config file assume corrent format
# called at 1st menu hit

proc config_menu file {
	if {![file exists $file]} {return ""}
	catch {destroy .menu}
	set fd [open $file r]
	menu .menu
	while {! [eof $fd]} {
		gets $fd line
		if {[string length $line] < 2 || [string index $line 0] == "#"} continue
		.menu add command -label [lindex $line 0] \
				-command "go_menu \{[lrange $line 1 end]\}"
	}
	close $fd
	return .menu
}

# post the channel menu (bound to Shift-button-2 on channel item)

proc  proc_menu {chan x y} {
	global prop curr_channel
	if {[info commands .menu] == "" && [config_menu $prop(config)]==""} {
		do_message "no config file"
	} else {
		incr chan
		set curr_channel $chan
		.menu post $x $y
	}
}

# command run when menu item is selected

proc go_menu cmd {
	global curr_channel			;# current channel set in proc_menu
	remote "v=$curr_channel;$cmd" "setting voice"
	.menu unpost
}
	
# process channel sliders

proc v_gain {chan value} {
	queue "v=$chan;g=$value"
	}

# layout procedures

proc do_label {win name pos {var ""}} {
	global prop	
	if {"$win" == "."} {set base ""} {set base $win}
	label $base.${name}_label  -text $name  \
		-relief $prop(relief) -border $prop(border) \
		-width [string length $name]
	if {$var != ""} {$base.${name}_label configure -textvariable $var}
	blt_table $win $base.${name}_label $pos -fill both
	}

proc do_scale {win name pos min init max} {
	global prop
	if {"$win" == "."} {set base ""} {set base $win}
	scale $base.$name -orient horizontal -from $min -to $max -showvalue false \
		-relief $prop(relief) -border $prop(border) \
		-command "slide_command $name" -width 10
	$base.$name set $init
	blt_table $win $base.$name $pos -fill both
	}

# short-cut - set channel opts (not done yet)

proc opts chan {
	puts "opts, chan $chan"
	if {[catch "toplevel .t$chan"]} return
	wm title .t$chan "channel $chan options"
	label .t$chan.lab -text "voicing parameters for channel $chan"
	blt_table .t$chan .t$chan.lab 0,0 -columnspan 3
	do_label .t$chan cutoff 1,0 
	do_scale .t$chan cutoff 1,1 100 1000 4000
	do_label .t$chan 001 1,2 cutoff

	do_label .t$chan inharmonicity 2,1
	do_scale .t$chan stretch 2,1 -5 0 40
	do_label .t$chan 002 2,2 inharmonicity
	focus .t$chan
	bind .t$chan <Any-Leave> "destroy .t$chan"
}

proc stack {win list {how vert}} {
	global prop
	if {"$win" == "."} {set base ""} {set base $win}
	set indx 0
	foreach i $list {
		frame $base.$i -relief flat -border $prop(border)
		blt_table $win $base.$i $indx,0 -fill both
		incr indx
		}
	}

proc slide_command {name value} {
	global $name cmd
	set $name $value
	queue $cmd($name) $value
	}

proc but {win var but} {
	global chan$var chan

	# toggle state

	if {$but == "2"} {
		if {$chan($var) == "on"} {
			set relief raised; set chan($var) "off";queue "v=$var;b=-"
		} else {
			set relief sunken; set chan($var) "on";queue "v=$var;b=[set chan$var]"
		}
	$win configure -relief $relief
	return
	}
		
	if {$but == "1"} {set list "< + >"} {set list "> + <"}
	set foo [lindex "[lindex $list 0] $list" [lsearch $list [set chan$var]]]
	set chan$var $foo
	queue "v=$var;b=$foo"
}

proc do_incr {win var amount} {
	global $var global
	$win.$var set [incr $var $amount]
	set $var [$win.$var get]
	}

proc reset_buttons {but} {
	foreach i {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16} {
		global chan$i chan
		set mode [lindex "x < + >" $but]
		set chan$i $mode
		set chan($i) "on"
		after [expr $i * 100] "queue \"v=$i;b=$mode\""	;# synth bug
		.main.chan.$i.x configure -relief sunken
		.main.chan.$i.s set 5
		}
	}

# set binding to highlight "active" labels

proc highlight {win color} {
	bind $win <Any-Enter> "%W configure -fg $color"
	bind $win <Any-Leave> "%W configure -fg black"
}

# build main window partitions

stack . "top main bottom"
blt_table row . configure 0 -resize none
blt_table row . configure 2 -resize none

# fill in the sliders

set indx 0
foreach i {volume style transposition tempo intensity} {
	do_label .main $i $indx,0
	eval "do_scale .main $i $indx,1  $limits($i)"
	.main.$i set [lindex $limits($i) 1]
	do_label .main 00$indx $indx,2 $i
	bind .main.00${indx}_label <Button-3> "do_incr .main $i 1"
	bind .main.00${indx}_label <Button-1> "do_incr .main $i -1"
	bind .main.00${indx}_label <Button-2> \
		"set $i [lindex $limits($i) 1]; .main.$i set \$$i"
	incr indx
	}

# make the channel select sliders (exp version)

do_label .main channels 6,0
do_label .main reset 6,2
highlight .main.reset_label red
bind .main.reset_label <ButtonPress> "reset_buttons %b"
frame .main.chan -relief $prop(relief) -border $prop(border)
foreach i {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16} {
	frame .main.chan.$i -bg red -border 1
	label .main.chan.$i.x -width 1 -textvariable chan$i \
		-relief sunken -border 1
	scale .main.chan.$i.s -width 5 -orient vertical -from 9 -to 1 \
		-length 45 -sliderlength 7 -border 1 -sliderforeground red \
		-showvalue false -relief sunken -command "v_gain $i"
	.main.chan.$i.s set 5
	highlight .main.chan.$i.x red
	bind .main.chan.$i.x <ButtonPress> "but %W $i %b"
	bind .main.chan.$i.x <Shift-2> "proc_menu $i %X %Y"
	bind .main.chan.$i.x <Double-2> "opts $indx"
	bind .main.chan.$i.s <2> ".main.chan.$i.s set 5"
	global chan$i
	set chan($i) "on"
	set chan$i "+"
	blt_table .main.chan .main.chan.$i 0,$i -fill both
	blt_table .main.chan.$i .main.chan.$i.x 0,0 -fill none
	if {$verbose == "on"} {			# show channel volumes
		blt_table .main.chan.$i .main.chan.$i.s 1,0 -fill none
		}
	}
blt_table .main .main.chan 6,1 -fill both

# do the top and bottom stuff

set indx 0
foreach i {file status note } {
	label .top.$i -text $i
	blt_table .top .top.$i 0,$indx -fill both
	if {$i == "file"} {
		entry .top.x$i -textvariable $i -relief $prop(relief) \
				-border $prop(border)
	} else {
		label .top.x$i -textvariable $i -relief $prop(relief) \
				-border $prop(border)
	}
	blt_table .top .top.x$i 1,$indx -fill both
	incr indx
	}
.top.xnote configure -width 5
.top.xstatus configure -relief raised
highlight .top.xstatus red
bind .top.xstatus <Button-1> "toggle_status %W"
bind .top.xstatus <ButtonRelease-1> "%W configure -relief raised"

# file entry 

focus default .top.xfile
focus .top.xfile
.top.xfile configure -width 20
bind .top.xfile <Return> get_file
bind .top.xfile <Tab> "comp_file .top.xfile"
bind .top.xfile <1> {set file "";comp_file .top.xfile}

button .top.quit -text quit -command "die {}" -activebackground pink
blt_table .top .top.quit 0,3 -rowspan 2 -fill both

label .bottom.msg -textvariable message
blt_table .bottom .bottom.msg 0,0 -fill both

# global bindings

bind . <Control-c> "destroy ."

# startup
set control [open "|$synth" w+]
update
do_message "Setting Current directory to $prop(dir)"
# do_message "started synth, process [pid $control]"
queue h 3000
cycle
